<?php
/**
 * AmazonSearch MediaWiki extension  version 1.0.1
 *	for details please see: https://kimagurenote.net/kn/AmazonSearch
 *
 * Copyright (c) 2019 Kimagurenote https://kimagurenote.net/
 * License: Revised BSD license http://opensource.org/licenses/BSD-3-Clause
 *
 * Function:
 *	Mediawiki extension that make item links for several shopping site.
 *
 * Usage:
 *	<amazon>free keyword</amazon> on your wiki page.
 *	for details please see: https://kimagurenote.net/kn/AmazonSearch
 *
 * History:
 * 2019.06.20 Version 1.0.1
 *	bypass mbstring if not defined.
 * 2019.06.20 Version 1.0.0
 *	1st release
 *
 * @file
 * @ingroup Extensions
 * @author Kimagurenote
 * @copyright © 2019 Kimagurenote
 * @license The BSD 3-Clause License
 */

if( !defined( 'MEDIAWIKI' ) ) {
	echo( "This file is an extension to the MediaWiki software and cannot be used standalone.\n" );
	die( 1 );
}

$wgExtensionCredits['parserhook'][] = array(
	'path'           => __FILE__,
	'name'           => 'AmazonSearch',
	'author'         => array(
		'[https://kimagurenote.net/ Kimagurenote]'
	),
	'url'            => 'https://kimagurenote.net/kn/AmazonSearch',
	'version'        => '1.0.1',
	'license-name'   => 'BSD-3-Clause',
	'descriptionmsg' => 'amazonsearch-desc'
);

/* Internationalization */
$wgMessagesDirs['AmazonSearch'] = __DIR__ . '/i18n';

/* default values */
#define( 'AMAZONSEARCH_DEBUG', 'amazonsearch' );	/* for debug */
define( 'AMAZONSEARCH_HOOKID', 'amazon' );			/* tag name */
define( 'AMAZONSEARCH_CLASS_DIV', 'amazonsearch' );	/* html div class name */

/* query type identifier */
define( 'AMAZONSEARCH_QUERYTYPE_ISBN', 'isbn' );
define( 'AMAZONSEARCH_QUERYTYPE_EAN', 'ean' );

/* operation parameter */
define( 'AMAZONSEARCH_NAMEMAXLEN', -1 );
define( 'AMAZONSEARCH_AUTHORMAXLEN', -1 );

/* Product Advertising API URL list
 * https://images-na.ssl-images-amazon.com/images/G/09/associates/paapi/dg/index.html?CHAP_Intro_AAWS.html
 */
define( 'AMAZONSEARCH_APIURL_METHOD', 'GET' );
define( 'AMAZONSEARCH_APIURL_PROTOCOL', 'http://' );
define( 'AMAZONSEARCH_APIURL_PATH', '/onca/xml' );
$wgAmazonSearchAPIDomain['ca'] = 'ecs.amazonaws.ca';
$wgAmazonSearchAPIDomain['de'] = 'ecs.amazonaws.de';
$wgAmazonSearchAPIDomain['fr'] = 'ecs.amazonaws.fr';
$wgAmazonSearchAPIDomain['jp'] = 'ecs.amazonaws.jp';
$wgAmazonSearchAPIDomain['uk'] = 'ecs.amazonaws.co.uk';
$wgAmazonSearchAPIDomain['us'] = 'ecs.amazonaws.com';
# ca/de/fr/jp/uk/us
$wgAmazonSearchDefaultRegion = 'jp';

/* tag classes */
$wgAmazonSearchClass['div'] = '';
$wgAmazonSearchClass['img'] = 'amazonsearchimg';
$wgAmazonSearchClass['name'] = 'amazonsearchname';
$wgAmazonSearchClass['author'] = 'amazonsearchauthor';
$wgAmazonSearchClass['buttons'] = 'amazonsearchbuttons';
$wgAmazonSearchClass['button'] = 'shoplinkamazon';

/* accepted parameters for pass to API */
$wgAmazonSearchAcceptArgs = array( 'Actor', 'Artist', 'AudienceRating', 'Author', 'Brand', 'BrowseNode', 'Composer', 'Condition', 'Conductor', 'Director', 'Manufacturer', 'MaximumPrice', 'MerchantId', 'MinimumPrice', 'Orchestra', 'Publisher', 'SearchIndex', 'Sort' );


/* MediaWiki set hook */
$wgHooks['ParserFirstCallInit'][] = 'wfAmazonSearchSetHook';

function wfAmazonSearchSetHook( $parser ) {
	$parser->setHook( AMAZONSEARCH_HOOKID, 'wfRenderAmazonSearch' );
	return true;
}

/* Main process - The callback function for embed shopping link
 *
 */
function wfRenderAmazonSearch( $value, $argv, $parser ) {
	global $wgAmazonSearchAPIDomain, $wgAmazonSearchAPIKey, $wgAmazonSearchAPISecret, $wgAmazonSearchAssociateTag, $wgAmazonSearchDefaultRegion, $wgAmazonSearchDefaultMerchantId, $wgAmazonSearchAcceptArgs, $wgAmazonSearchClass, $wgAmazonSearchMaxLen;

	/* for debug */	AMAZONSEARCH_DEBUG && ( $parser->disableCache() );

	/* search keyword */
	$value = wfAmazonSearchConvertEncoding(trim($value), 'UTF-8');

	/* require apiurl, appid and keyword(value) */
	if( empty($wgAmazonSearchAPIKey) || empty($wgAmazonSearchAPISecret) || empty($value) ) {
		/* for debug */	AMAZONSEARCH_DEBUG && ( wfDebugLog( AMAZONSEARCH_DEBUG, "wfRenderAmazonSearch(): some required parameter(s) empty.\n" ) );
		return '<!-- amazon>some required parameter(s) empty.</amazon -->';
	}

	/* region select */
	$region = (@$argv['region']) ? @$argv['region'] : $wgAmazonSearchDefaultRegion;
	if( empty($wgAmazonSearchAPIDomain[$region]) ) {
		/* for debug */	AMAZONSEARCH_DEBUG && ( wfDebugLog( AMAZONSEARCH_DEBUG, "wfRenderAmazonSearch(): Region $region not defined.\n" ) );
		return '<!-- amazon>Region ' . htmlentities($region) . ' not defined.</amazon -->';
	}

	/* generate API parameters */
	$apiparam['Service'] = 'AWSECommerceService';
	$apiparam['AWSAccessKeyId'] = isset($wgAmazonSearchAPIKey[$region]) ? $wgAmazonSearchAPIKey[$region] : $wgAmazonSearchAPIKey;
	$apisecret = isset($wgAmazonSearchAPISecret[$region]) ? $wgAmazonSearchAPISecret[$region] : $wgAmazonSearchAPISecret;
	$atag = $apiparam['AssociateTag'] = isset($wgAmazonSearchAssociateTag[$region]) ? $wgAmazonSearchAssociateTag[$region] : $wgAmazonSearchAssociateTag;
	$apiparam['Operation'] = 'ItemSearch';
	$apiparam['ResponseGroup'] = 'Small,Images';
	$apiparam['Version'] = '2010-09-01';
	$apiparam['SearchIndex'] = 'All';
	/* if value seems ISBN */
	if( preg_match('/978\-[\d\-]+/', $value) ) {
		$value = str_replace('-', '', $value);
	}
	$apiparam['Keywords'] = $value;
	/* Availability (option)
	 * https://images-na.ssl-images-amazon.com/images/G/09/associates/paapi/dg/OffersVersusAvailability.html
	 */
	$apiparam['Availability'] = 'Available';
	$apiparam['Condition'] = 'All';
	$apiparam['MerchantId'] = empty($wgAmazonSearchDefaultMerchantId) ? 'All' : $wgAmazonSearchDefaultMerchantId;

	/* override/more parameters if accepted (option) */
	if( !empty(@$argv) ) {
		foreach( $wgAmazonSearchAcceptArgs as $v ) {
			if( !empty(@$argv[strtolower($v)]) ) {
				$apiparam[$v] = @$argv[strtolower($v)];
				/* for debug */	AMAZONSEARCH_DEBUG && ( wfDebugLog( AMAZONSEARCH_DEBUG, "wfRenderAmazonSearch(): override parameter $v = " . $apiparam[$v] . "\n" ) );
			}
		}
	}

	/* generate URL */
	if( empty( $apiurl = wfAmazonSearchGenerateAPISign($wgAmazonSearchAPIDomain[$region], $apiparam, $apisecret) ) ) {
		/* for debug */	AMAZONSEARCH_DEBUG && ( wfDebugLog( AMAZONSEARCH_DEBUG, "wfRenderAmazonSearch(): can't generate api url.\n" ) );
		return '<!-- amazon>API URL generate failure.</amazon -->';
	}

	/* call API */
	if( ($xml = file_get_contents($apiurl)) === false ) {
		/* for debug */	AMAZONSEARCH_DEBUG && ( wfDebugLog( AMAZONSEARCH_DEBUG, "wfRenderAmazonSearch(): can't call api $apiurl.\n" ) );
		return '<!-- amazon>API call failure.</amazon -->';
	}

	/* for debug */	AMAZONSEARCH_DEBUG && ( wfDebugLog( AMAZONSEARCH_DEBUG, "wfRenderAmazonSearch(): API call success.\n$apiurl\n" ) );

	/* decode XML */
	if( ($xml = simplexml_load_string( wfAmazonSearchConvertEncoding( $xml, 'UTF-8') ) ) === false ) {
		/* for debug */	AMAZONSEARCH_DEBUG && ( wfDebugLog( AMAZONSEARCH_DEBUG, "wfRenderAmazonSearch(): can't decode xml:\n$xml\n" ) );
		return '<!-- amazon>xml decode failure.</amazon -->';
	}
	$json = json_encode($xml);
	if( is_null( $resapi = json_decode( $json, true ) ) ) {
		/* for debug */	AMAZONSEARCH_DEBUG && ( wfDebugLog( AMAZONSEARCH_DEBUG, "wfRenderAmazonSearch(): can't decode json:\n$json\n" ) );
		return '<!-- amazon>json decode failure.</amazon -->';
	}

	/* no result */
	if( $resapi['Items']['TotalResults'] < 1 ) {
		/* for debug */	AMAZONSEARCH_DEBUG && ( wfDebugLog( AMAZONSEARCH_DEBUG, "wfRenderAmazonSearch(): no result.\n" . var_export($resapi, TRUE) . "\n" ) );
		return '<!-- amazon>' . htmlentities($resapi['Error']['Message']) . '</amazon -->';
	}

	/* get result, render start */
	$item = $resapi['Items']['Item'];
	if( isset($item[0]) ) {
		$item = $item[0];
	}
	/* for debug */	AMAZONSEARCH_DEBUG && wfDebugLog( AMAZONSEARCH_DEBUG, "wfRenderAmazonSearch(): get result, first item:\n" . var_export($item, TRUE) . "\n" );

	/* generate embed HTML tags */
	$ret = '<div class="' . AMAZONSEARCH_CLASS_DIV;
	if( !empty($wgAmazonSearchClass['div']) ) {
		$ret .= ' ' . $wgAmazonSearchClass['div'];
	}
	$ret .= '">';

	/* image */
	$ret .= '<a href="' . $item['DetailPageURL'] . '" target="_blank" rel="nofollow">';
	$ret .= '<img src="' . $item['MediumImage']['URL'];
	if( !empty($wgAmazonSearchClass['img']) ) {
		$ret .= '" class="' . $wgAmazonSearchClass['img'];
	}
	$ret .= '" /></a>';

	/* item name and link */
	$name = $item['ItemAttributes']['Title'];
	$maxlen = $wgAmazonSearchMaxLen['name'] ? $wgAmazonSearchMaxLen['name'] : AMAZONSEARCH_NAMEMAXLEN;
	$ret .= '<p';
	if( !empty($wgAmazonSearchClass['name']) ) {
		$ret .= ' class="' . $wgAmazonSearchClass['name'] . '"';
	}
	$ret .= '><a href="' . $item['DetailPageURL'] . '" target="_blank" rel="nofollow">';
	$ret .= htmlspecialchars(wfAmazonSearchStrimwidth($name, 0, $maxlen));
	$ret .= '</a></p>';

	/* Author, Artist, Actor or Manufacturer name */
	if( !empty($text = $item['ItemAttributes']['Author']) ) {
		if( !empty($item['ItemAttributes']['Creator']) ) {
			$text .= ' / ' . $item['ItemAttributes']['Creator'];
		}
	} elseif(  empty($text = $item['ItemAttributes']['Artist'])
			&& empty($text = $item['ItemAttributes']['Actor']) ) {
		$text = $item['ItemAttributes']['Manufacturer'];
	}
	$maxlen = $wgAmazonSearchMaxLen['author'] ? $wgAmazonSearchMaxLen['author'] : AMAZONSEARCH_AUTHORMAXLEN;
	$ret .= '<p';
	if( !empty($wgAmazonSearchClass['author']) ) {
		$ret .= ' class="' . $wgAmazonSearchClass['author'] . '"';
	}
	$ret .= '>';
	$ret .= htmlspecialchars(wfAmazonSearchStrimwidth($text, 0, $maxlen));
	$ret .= '</p>';

	/* link button */
	if( !empty( $caption = wfMessage( 'amazonsearch-sitename-' . $region )->inContentLanguage()->escaped() ) ) {
		$ret .= '<ul';
		if( !empty($wgAmazonSearchClass['buttons']) ) {
			$ret .= ' class="' . $wgAmazonSearchClass['buttons'] . '"';
		}
		$ret .= '><li';
		if( !empty($wgAmazonSearchClass['button']) ) {
			$ret .= ' class="' . $wgAmazonSearchClass['button'] . '"';
		}
		$ret .= '><a href="https://www.amazon.co.jp/gp/search?keywords=' . rawurlencode($value) . '&__mk_ja_JP=%E3%82%AB%E3%82%BF%E3%82%AB%E3%83%8A&tag=' . $atag . '" title="' . wfMessage( 'amazonsearch-button-title', $value, $caption )->inContentLanguage()->escaped() . '" target="_blank" rel="nofollow">' . wfMessage( 'amazonsearch-button-caption', $caption )->inContentLanguage()->escaped() . '</a></li></ul>';
	}
	$ret .= "</div>\n";

	/* for debug */	AMAZONSEARCH_DEBUG && ( wfDebugLog( AMAZONSEARCH_DEBUG, "wfRenderYJShopping() complete: \n$ret\n" ) );

	return array( $ret, "markerType" => 'nowiki' );
}


/* for generate Amazon API URL with Signature
 *
 */
function wfAmazonSearchGenerateAPISign( $domain, $params, $key ) {
	/* incomplete parameters */
	if( empty($params) || empty($key) ) {
		return '';
	}

	/* generate Timestamp */
	date_default_timezone_set('UTC');
	$params['Timestamp'] = date('Y-m-d\TH:i:s\Z');

	$data = AMAZONSEARCH_APIURL_METHOD . "\n$domain\n" . AMAZONSEARCH_APIURL_PATH . "\n";
	$url = AMAZONSEARCH_APIURL_PROTOCOL . $domain . AMAZONSEARCH_APIURL_PATH . '?';

	ksort($params);
	$query = http_build_query($params, '', '&', PHP_QUERY_RFC3986);
	if( !empty($query) ) {
		$data .= $query;
		$url .= $query;

		$url .= '&Signature=' . rawurlencode( base64_encode( hash_hmac('sha256', $data, $key, true) ) );

		return $url;
	} else {
		return '';
	}
}


/* mb_convert_encoding() wrapper
 */
function wfAmazonSearchConvertEncoding( $str, $to_encoding )  {
	if( ini_get( 'mbstring.detect_order' ) ) {
		return mb_convert_encoding($str, $to_encoding);
	} else {
		return $str;
	}
}

/* mb_strimwidth() wrapper
 */
function wfAmazonSearchStrimwidth( $str, $start, $width )  {
	if( $width < 0 || strlen($str) <= $width ) {
		return $str;
	} else {
		if( ini_get( 'mbstring.detect_order' ) ) {
			return mb_strimwidth($str, $start, $width, wfMessage( 'amazonsearch-trimmarker' )->inContentLanguage()->plain() );
		} else {
			return substr($str, $start, $width) . wfMessage( 'amazonsearch-trimmarker' )->inContentLanguage()->plain();
		}
	}
}

?>
